# ../addons/eventscripts/gungame/scripts/custom/gg_twl_win/gg_twl_win.py

# ============================================================================
# >> IMPORTS
# ============================================================================
# Eventscripts Imports
import es
import gamethread
from playerlib import getPlayer

# GunGame Imports
from gungame51.core.addons.shortcuts import AddonInfo
from gungame51.core.addons import load as addon_load
from gungame51.core.addons import unload as addon_unload
from gungame51.core.addons.shortcuts import get_loaded_addon_list
from gungame51.core.players import Player
from gungame51.core.players.shortcuts import add_attribute_callback
from gungame51.core.players.shortcuts import remove_callbacks_for_addon
from gungame51.core.messaging.shortcuts import saytext2
from gungame51.core.messaging.shortcuts import centermsg
from gungame51.core.messaging.shortcuts import toptext
from gungame51.scripts.included.gg_warmup_round.gg_warmup_round import \
                                                                    do_warmup

# ============================================================================
# >> ADDON REGISTRATION/INFORMATION
# ============================================================================
info = AddonInfo()
info.name = "gg_twl_win"
info.title = "GG TWL Win"
info.author = "Warren Alpert"
info.version = "0.1"

# ============================================================================
# >> GLOBAL VARIABLES
# ============================================================================
gg_warmup_round = es.ServerVar("gg_warmup_round")
gg_warmup_round_backup = int(gg_warmup_round)

playerWon = False
winnerWarmupOver = False

# ============================================================================
# >> LOAD & UNLOAD
# ============================================================================
def load():
    add_attribute_callback('wins', wins_call_back, info.name)
    es.dbgmsg(0, 'Loaded: %s' % info.name)

def unload():
    es.dbgmsg(0, 'Unloaded: %s' % info.name)
    remove_callbacks_for_addon(info.name)

# ============================================================================
# >> GAME EVENTS
# ============================================================================
def es_map_start(event_var):
    global playerWon, winnerWarmupOver
    playerWon = False
    winnerWarmupOver = False

def gg_start(event_var):
    global winnerWarmupOver
    # If the player has won during this map, warmup just ended, so end the map
    if playerWon:
        winnerWarmupOver = True
        userid = es.getuserid()
        # If there is a player on the server to let us end the map with
        if userid:
            es.server.queuecmd("es_xgive %s game_end" % userid)
            es.server.queuecmd("es_xfire %s game_end EndGame" % userid)
            # If warmup is unloaded by default, unload it
            if not gg_warmup_round_backup:
                addon_unload("gg_warmup_round")
                gg_warmup_round.set(0)
    else:
        winnerWarmupOver = False

def player_spawn(event_var):
    # If the player has won during this map
    if playerWon:
        userid = event_var["userid"]
        # If the warmup round caused by the win is over, end the map
        # This only happens if there were no players on the server when warmup
        # ended.
        if winnerWarmupOver:
            es.server.queuecmd("es_xgive %s game_end" % userid)
            es.server.queuecmd("es_xfire %s game_end EndGame" % userid)
            # If warmup is unloaded by default, unload it
            if not gg_warmup_round_backup:
                addon_unload("gg_warmup_round")
                gg_warmup_round.set(0)
        # If the player is spawning in between the win and the warmup, freeze
        # them
        else:
            pPlayer = getPlayer(userid)
            if not pPlayer.isdead:
                pPlayer.freeze = 1
                pPlayer.godmode = 1

# ============================================================================
# >> CUSTOM/HELPER FUNCTIONS
# ============================================================================
def wins_call_back(name, value, ggPlayer):
    global playerWon
    # A player won
    playerWon = True

    # Get player info
    userid = ggPlayer.userid
    index = getPlayer(userid).index
    name = es.getplayername(userid)
    team = es.getplayerteam(userid)

    # Tell the world
    saytext2('#human', index, 'PlayerWon', {'player':name})

    # Play the winner sound, and freeze the players
    for playerid in es.getUseridList():
        pPlayer = getPlayer(playerid)
        if not pPlayer.isdead:
            pPlayer.freeze = 1
            pPlayer.godmode = 1

        if not es.isbot(playerid):
            Player(playerid).playsound('winner')

    # Tell the world (center message)
    centermsg('#human', 'PlayerWon_Center', {'player': name})
    gamethread.delayed(1, centermsg, ('#human', 'PlayerWon_Center', 
                                                    {'player': name}))
    gamethread.delayed(2, centermsg, ('#human', 'PlayerWon_Center', 
                                                    {'player': name}))
    gamethread.delayed(3, centermsg, ('#human', 'PlayerWon_Center', 
                                                    {'player': name}))

    # Toptext
    if team == 2:
        toptext('#human', 10, '#red', 'PlayerWon_Center', 
                                                    {'player': name})
    else:
        toptext('#human', 10, '#blue', 'PlayerWon_Center', 
                                                    {'player': name})

    # If warmup is loaded, do_warmup in 6 seconds
    if "gg_warmup_round" in get_loaded_addon_list():
        gg_warmup_round_backup = 1
        gamethread.delayed(6, do_warmup)
    # If warmup is not loaded, load it in 6 seconds
    else:
        gg_warmup_round_backup = 0
        gamethread.delayed(6, addon_load, "gg_warmup_round")
        gg_warmup_round.set(1)
    # Restart the game in 2 seconds from warmup starting
    # We use gamethread for the delay because it is a more precise delay than
    # mp_restartgame's delay
    gamethread.delayed(6, es.server.queuecmd, "mp_restartgame 2")
    raise AttributeError('gg_cancel_callback')